unit FreeOTFEfmeOptions_General;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, FreeOTFEfmeOptions_Base, StdCtrls, ExtCtrls, Spin64;

type
  TfmeOptions_General = class(TfmeOptions_Base)
    gbGeneral: TGroupBox;
    Label3: TLabel;
    ckAllowMultipleInstances: TCheckBox;
    ckDisplayToolbar: TCheckBox;
    ckDisplayStatusbar: TCheckBox;
    cbDragDrop: TComboBox;
    ckAutoStartPortable: TCheckBox;
    ckAdvancedMountDlg: TCheckBox;
    ckExploreAfterMount: TCheckBox;
    ckRevertVolTimestamps: TCheckBox;
    pnlVertSplit: TPanel;
    cbDrive: TComboBox;
    Label1: TLabel;
    seMRUMaxItemCount: TSpinEdit64;
    Label4: TLabel;
    Label5: TLabel;
    procedure FrameResize(Sender: TObject);
  private
    { Private declarations }
  public
    procedure Initialize(); override;
    procedure ReadSettings(); override;
    procedure WriteSettings(); override;
  end;

implementation

{$R *.dfm}

uses
  SDUGeneral,
  FreeOTFESettings;

resourcestring
  USE_DEFAULT = 'Use default';

procedure TfmeOptions_General.FrameResize(Sender: TObject);
begin
  SDUCenterControl(gbGeneral, ccHorizontal);
  SDUCenterControl(gbGeneral, ccVertical, 25);
end;

procedure TfmeOptions_General.Initialize();
var
  driveLetter: char;
begin
  inherited;

  pnlVertSplit.caption := '';
  pnlVertSplit.bevelouter := bvLowered;
  pnlVertSplit.width := 3;

  cbDrive.Items.clear();
  cbDrive.Items.Add(USE_DEFAULT);
  for driveLetter:='C' to 'Z' do
    begin
    cbDrive.Items.Add(driveLetter+':');
    end;

end;

procedure TfmeOptions_General.ReadSettings();
var
  ft: TDragDropFileType;
  idx: integer;
  useIdx: integer;
begin
  // General...
  ckExploreAfterMount.checked := Settings.OptExploreAfterMount;
  ckAllowMultipleInstances.checked := Settings.OptAllowMultipleInstances;
  ckDisplayToolbar.checked := Settings.OptDisplayToolbar;
  ckDisplayStatusbar.checked := Settings.OptDisplayStatusbar;
  ckAutoStartPortable.checked := Settings.OptAutoStartPortable;
  ckAdvancedMountDlg.checked := Settings.OptAdvancedMountDlg;
  ckRevertVolTimestamps.checked := Settings.OptRevertVolTimestamps;

  // Populate and set drag drop filetype dropdown
  cbDragDrop.Items.Clear();
  idx := -1;
  useIdx := -1;
  for ft:=low(DragDropFileTypeTitle) to high(DragDropFileTypeTitle) do
    begin
    inc(idx);
    cbDragDrop.Items.Add(DragDropFileTypeTitle[ft]);
    if (Settings.OptDragDropFileType = ft) then
      begin
      useIdx := idx;
      end;
    end;
  cbDragDrop.ItemIndex := useIdx;

  // Default drive letter
  if (Settings.OptDefaultDriveLetter = #0) then
    begin
    cbDrive.ItemIndex := 0;
    end
  else
    begin
    cbDrive.ItemIndex := cbDrive.Items.IndexOf(Settings.OptDefaultDriveLetter+':');
    end;

  seMRUMaxItemCount.Value := Settings.OptMRUList.MaxItems;

end;


procedure TfmeOptions_General.WriteSettings();
var
  ft: TDragDropFileType;
begin
  // General...
  Settings.OptExploreAfterMount := ckExploreAfterMount.checked;
  Settings.OptAllowMultipleInstances := ckAllowMultipleInstances.checked;
  Settings.OptDisplayToolbar := ckDisplayToolbar.checked;
  Settings.OptDisplayStatusbar := ckDisplayStatusbar.checked;
  Settings.OptAutoStartPortable := ckAutoStartPortable.checked;
  Settings.OptAdvancedMountDlg := ckAdvancedMountDlg.checked;
  Settings.OptRevertVolTimestamps := ckRevertVolTimestamps.checked;

  // Decode drag drop filetype
  Settings.OptDragDropFileType := ftPrompt;
  for ft:=low(DragDropFileTypeTitle) to high(DragDropFileTypeTitle) do
    begin
    if (DragDropFileTypeTitle[ft] = cbDragDrop.Items[cbDragDrop.ItemIndex]) then
      begin
      Settings.OptDragDropFileType := ft;
      break;
      end;
    end;

  // Default drive letter
  if (cbDrive.ItemIndex = 0) then
    begin
    Settings.OptDefaultDriveLetter := #0;
    end
  else
    begin
    Settings.OptDefaultDriveLetter := cbDrive.Items[cbDrive.ItemIndex][1];
    end;

  Settings.OptMRUList.MaxItems := seMRUMaxItemCount.Value;

end;

END.

