unit FreeOTFEfmeOptions_PKCS11;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, FreeOTFEfmeOptions_Base, StdCtrls, FreeOTFEfmeVolumeSelect,
  OTFEFreeOTFE_U;

type
  TfmeOptions_PKCS11 = class(TfmeOptions_Base)
    gbPKCS11: TGroupBox;
    lblLibrary: TLabel;
    edLibFilename: TEdit;
    pbBrowse: TButton;
    OpenDialog1: TOpenDialog;
    ckEnablePKCS11: TCheckBox;
    pbVerify: TButton;
    gbPKCS11AutoActions: TGroupBox;
    lblAutoMountVolume: TLabel;
    ckPKCS11AutoDismount: TCheckBox;
    ckPKCS11AutoMount: TCheckBox;
    fmeAutoMountVolume: TfmeVolumeSelect;
    procedure FrameResize(Sender: TObject);
    procedure pbBrowseClick(Sender: TObject);
    procedure pbVerifyClick(Sender: TObject);
    procedure ControlChanged(Sender: TObject);
  private
    function LibraryDLL(): string;
    function VerifyLibrary(): boolean;

  public
    OTFEFreeOTFE: TOTFEFreeOTFE;

    procedure EnableDisableControls(); override;
    procedure Initialize(); override;
    procedure ReadSettings(); override;
    function CheckSettings(): boolean; override;
    procedure WriteSettings(); override;
  end;

implementation

{$R *.dfm}

uses
  SDUGeneral,
  SDUDialogs,
  FreeOTFESettings,
  pkcs11_library;


procedure TfmeOptions_PKCS11.ControlChanged(Sender: TObject);
begin
  inherited;
  EnableDisableControls();

end;

procedure TfmeOptions_PKCS11.FrameResize(Sender: TObject);
begin
  SDUCenterControl(gbPKCS11, ccHorizontal);
  SDUCenterControl(gbPKCS11, ccVertical, 25);
end;

procedure TfmeOptions_PKCS11.pbBrowseClick(Sender: TObject);
begin
  inherited;

  SDUOpenSaveDialogSetup(OpenDialog1, edLibFilename.text);
  if OpenDialog1.Execute() then
    begin
    edLibFilename.text := OpenDialog1.Filename;
    end;

end;

procedure TfmeOptions_PKCS11.pbVerifyClick(Sender: TObject);
begin
  inherited;
  
  if VerifyLibrary() then
    begin
    SDUMessageDlg('PKCS#11 library appears to be functional', mtInformation);
    end

end;

procedure TfmeOptions_PKCS11.Initialize();
begin
  OpenDialog1.DefaultExt := 'dll';
  OpenDialog1.Options := OpenDialog1.Options + [ofPathMustExist, ofFileMustExist, ofForceShowHidden];
  OpenDialog1.Filter := 'Library files (*.dll)|*.dll|All files|*.*';
  OpenDialog1.FilterIndex := 0;

  fmeAutoMountVolume.OTFEFreeOTFE := OTFEFreeOTFE;
  fmeAutoMountVolume.FileSelectFilter     := FILE_FILTER_FLT_VOLUMES;
  fmeAutoMountVolume.FileSelectDefaultExt := FILE_FILTER_DFLT_VOLUMES;

  fmeAutoMountVolume.OnChange := ControlChanged;
end;

procedure TfmeOptions_PKCS11.ReadSettings();
begin
  // General...
  ckEnablePKCS11.checked := Settings.OptPKCS11Enable;
  edLibFilename.text := Settings.OptPKCS11Library;

  ckPKCS11AutoMount.checked := Settings.OptPKCS11AutoMount;
  fmeAutoMountVolume.Filename := Settings.OptPKCS11AutoMountVolume;
  ckPKCS11AutoDismount.checked := Settings.OptPKCS11AutoDismount;

end;

procedure TfmeOptions_PKCS11.WriteSettings();
begin
  // General...
  Settings.OptPKCS11Enable := ckEnablePKCS11.checked;
  Settings.OptPKCS11Library := LibraryDLL;
  Settings.OptPKCS11AutoMount := ckPKCS11AutoMount.checked;
  Settings.OptPKCS11AutoMountVolume := fmeAutoMountVolume.Filename;
  Settings.OptPKCS11AutoDismount := ckPKCS11AutoDismount.checked;

end;

function TfmeOptions_PKCS11.LibraryDLL(): string;
begin
  Result := trim(edLibFilename.text);
end;

function TfmeOptions_PKCS11.VerifyLibrary(): boolean;
var
  lib: TPKCS11Library;
  allOK: boolean;
begin
  allOK := TRUE;
  if (LibraryDLL = '') then
    begin
    SDUMessageDlg('Please specify the PKCS#11 library to be used', mtError);
    allOK := FALSE;
    end;

  if allOK then
    begin
    allOK:= FALSE;
    try
      lib:= TPKCS11Library.Create(LibraryDLL);
      lib.Initialize();
      lib.Finalize();
      lib.Free();

      allOK := TRUE;
    except
      on E:Exception do
        begin
        // Nothing - just swallow it
        end;
    end;

    if not(allOK) then
      begin
      SDUMessageDlg('The library specified does not appear to be a valid/working PKCS#11 library', mtError);
      end;
    end;

  Result := allOK;
end;


function TfmeOptions_PKCS11.CheckSettings(): boolean;
var
  allOK: boolean;
begin
  allOK := inherited CheckSettings();

  if allOK then
    begin
    if ckEnablePKCS11.checked then
      begin
      allOK := VerifyLibrary();
      end;
    end;

  if ckEnablePKCS11.checked then
    begin
    if allOK then
      begin
      if ckPKCS11AutoMount.checked then
        begin
        if (fmeAutoMountVolume.Filename = '') then
          begin
          SDUMessageDlg(
                        'If automount on PKCS#11 token insertion is enabled, the volume to be mounted must be specified',
                        mtError
                       );
          allOK := FALSE;
          end;
        end;

      end;
    end;

  Result := allOK;
end;

procedure TfmeOptions_PKCS11.EnableDisableControls();
var
  pkcs11Enabled: boolean;
begin
  inherited;

  pkcs11Enabled := ckEnablePKCS11.checked;

  SDUEnableControl(pbBrowse,       pkcs11Enabled);
  SDUEnableControl(edLibFilename,  pkcs11Enabled);
  SDUEnableControl(pbVerify,       (
                                    pkcs11Enabled and
                                    (LibraryDLL <> '')
                                   ));


  SDUEnableControl(gbPKCS11AutoActions, pkcs11Enabled);
  SDUEnableControl(fmeAutoMountVolume, (
                                        pkcs11Enabled and  // Yes, this is needed; otherwise it'll enable/disable regardless
                                        ckPKCS11AutoMount.checked
                                       ));

end;

END.

