unit FreeOTFEfmeVolumeSelect;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons,
  OTFEFreeOTFE_U;

type
  TOpenSave = (fndOpen, fndSave);

  TfmeVolumeSelect = class(TFrame)
    bbBrowsePartition: TBitBtn;
    bbBrowseFile: TBitBtn;
    edFilename: TEdit;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    procedure bbBrowseFileClick(Sender: TObject);
    procedure bbBrowsePartitionClick(Sender: TObject);
    procedure edFilenameChange(Sender: TObject);
  private
    FFileButtonAdjust: integer;
    FSelectFor: TOpenSave;
    FFileSelectFilter: string;
    FFileSelectDefaultExt: string;
    FOnChange: TNotifyEvent;
    FFileGlyph: TBitmap;

    function  GetFilename(): string;
    procedure SetFilename(filename: string);
    function  GetAllowPartitionSelect(): boolean;
    procedure SetAllowPartitionSelect(allow: boolean);

  protected
    function  GetEnabled(): boolean; override;
    procedure SetEnabled(setValue: boolean); override;
  
  public
    OTFEFreeOTFE: TOTFEFreeOTFE;

    constructor Create(AOwner: TComponent); override;
    destructor  Destroy(); override;

    property Filename: string read GetFilename write SetFilename;
    property SelectFor: TOpenSave read FSelectFor write FSelectFor;
    property AllowPartitionSelect: boolean read GetAllowPartitionSelect write SetAllowPartitionSelect;

    property FileSelectFilter: string read FFileSelectFilter write FFileSelectFilter;
    property FileSelectDefaultExt: string read FFileSelectDefaultExt write FFileSelectDefaultExt;

  published
    property OnChange: TNotifyEvent read FOnChange write FOnChange;
  end;

implementation

{$R *.dfm}

uses
  SDUGeneral;
  
procedure TfmeVolumeSelect.bbBrowseFileClick(Sender: TObject);
var
  dlg: TOpenDialog; // Note: Save dialog inherits from this
begin
  dlg := SaveDialog;
  if (SelectFor = fndOpen) then
    begin
    dlg := OpenDialog;
    end;

  dlg.Filter     := FileSelectFilter;
  dlg.DefaultExt := FileSelectDefaultExt;

  SDUOpenSaveDialogSetup(dlg, edFilename.text);

  if dlg.Execute() then
    begin
    edFilename.text := dlg.Filename;
    end;

end;

procedure TfmeVolumeSelect.bbBrowsePartitionClick(Sender: TObject);
var
  selectedPartition: string;
begin
  if (OTFEFreeOTFE <> nil) then
    begin
    selectedPartition := OTFEFreeOTFE.SelectPartition();
    if (selectedPartition <> '') then
      begin
      edFilename.text := selectedPartition;
      end;
    end;

end;

constructor TfmeVolumeSelect.Create(AOwner: TComponent);
begin
  inherited;

  SetFilename('');

  self.height := edFilename.height;

  FFileGlyph := bbBrowseFile.Glyph;

  // Pre-calculate difference in file browse button position/TEdit width when
  // switching enabling/disabling partition select
  FFileButtonAdjust := (
                        (bbBrowseFile.Left + bbBrowseFile.Width) - // Position of righthand edge of browse file button
                        (edFilename.Left + edFilename.Width) // Position of righthand edge of filename TEdit
                       );

end;

destructor  TfmeVolumeSelect.Destroy();
begin
  // Return image, so it can be cleaned up as per normal
  bbBrowseFile.Glyph := FFileGlyph;

  inherited;
end;

function TfmeVolumeSelect.GetFilename(): string;
begin
  Result := Trim(edFilename.text);
end;

procedure TfmeVolumeSelect.SetFilename(filename: string);
begin
  edFilename.text := filename;
end;

function TfmeVolumeSelect.GetAllowPartitionSelect(): boolean;
begin
  Result := bbBrowsePartition.Enabled;
end;

procedure TfmeVolumeSelect.SetAllowPartitionSelect(allow: boolean);
begin
  if (bbBrowsePartition.Visible <> allow) then
    begin
    // Eliminate/allow partition button, remove/restore image from remaining
    // button; replace with "..." text if needed
    bbBrowsePartition.Visible := allow;

    if allow then
      begin
      bbBrowseFile.Caption := '';
      bbBrowseFile.Glyph := FFileGlyph;
      bbBrowseFile.Left := bbBrowseFile.Left - FFileButtonAdjust;
      edFilename.Width := edFilename.Width - FFileButtonAdjust;
      end
    else
      begin
      bbBrowseFile.Caption := '...';
      bbBrowseFile.Glyph := nil;
      bbBrowseFile.Left := bbBrowseFile.Left + FFileButtonAdjust;
      edFilename.Width := edFilename.Width + FFileButtonAdjust;
      end;
    end;

end;

procedure TfmeVolumeSelect.edFilenameChange(Sender: TObject);
begin
  if Assigned(OnChange) then
    begin
    OnChange(self);
    end;

end;

function TfmeVolumeSelect.GetEnabled(): boolean;
begin
  Result := inherited GetEnabled();
end;

procedure TfmeVolumeSelect.SetEnabled(setValue: boolean);
begin
  SDUEnableControl(edFilename, setValue);
  inherited;

  SDUEnableControl(bbBrowseFile, setValue);
  SDUEnableControl(bbBrowsePartition, setValue);
end;

END.

