unit FreeOTFEfrmGridReport_Cypher;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, FreeOTFEfrmGridReport, Menus, ActnList, StdCtrls, Grids, ExtCtrls,
  SDUStringGrid, Buttons, ComCtrls;

type
  TGridColumn_Cypher = (
                        gccDriverTitle,
                        gccDriverVersion,
                        gccDriverGUID,
                        gccDriverDeviceName,
                        gccDriverUserModeName,
                        gccDriverKernelModeName,

                        gccCypherTitle,
                        gccCypherKeysizeUnderlying,
                        gccCypherMode,
                        gccCypherVersion,
                        gccCypherKeysizeRequired,
                        gccCypherBlocksize,
                        gccCypherGUID
                       );

  TfrmGridReport_Cypher = class(TfrmGridReport)
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    // Get the grid column index for a specific column
    function  ColumnToColIdx(column: TGridColumn_Cypher): integer;
    function  ColIdxToColumn(colIdx: integer): TGridColumn_Cypher;

    procedure AddSubItem(col: TGridColumn_Cypher; item: TListItem; Value: string);

  public
    function CountDrivers(): integer; override;
    function CountImplementations(): integer; override;

    procedure SetupGrid(); override;
    procedure PopulateGrid(); override;
    function  IsColumnAdvanced(colIdx: integer): boolean; override;
  end;

implementation

{$R *.dfm}

uses
  SDUGeneral,
  SDUDialogs,
  OTFEFreeOTFE_U;

const
  TGridColumn_CypherTitle: array [TGridColumn_Cypher] of string = (
                                      'Driver title',
                                      'Driver version',
                                      'Driver GUID',
                                      'Driver device name',
                                      'Driver user mode name',
                                      'Driver kernel mode name',

                                      'Cypher title',
                                      'Cypher keysize',
                                      'Cypher mode',
                                      'Cypher version',
                                      'Cypher keysize required',
                                      'Cypher blocksize',
                                      'Cypher GUID'
                                     );

  ADVANCED_COLS_CYPHER: set of TGridColumn_Cypher = [
                                                    gccDriverGUID,
                                                    gccDriverDeviceName,
                                                    gccDriverUserModeName,
                                                    gccDriverKernelModeName,
                                                    gccCypherBlocksize,
                                                    gccCypherKeysizeRequired,
                                                    gccCypherGUID
                                                   ];



function TfrmGridReport_Cypher.ColumnToColIdx(column: TGridColumn_Cypher): integer;
begin
  Result:= ord(column);
end;

function TfrmGridReport_Cypher.ColIdxToColumn(colIdx: integer): TGridColumn_Cypher;
var
  i: TGridColumn_Cypher;
  retval: TGridColumn_Cypher;
begin
  retval := low(TGridColumn_Cypher);

  for i:=low(TGridColumn_Cypher) to high(TGridColumn_Cypher) do
    begin
    if (ColumnToColIdx(i) = colIdx) then
      begin
      retval := i;
      break;
      end;
    end;

  Result := retval;
end;

procedure TfrmGridReport_Cypher.AddSubItem(col: TGridColumn_Cypher; item: TListItem; Value: string);
var
  dispValue: string;
begin
  if (
      not(IsColumnAdvanced(ColumnToColIdx(col))) or
      ckShowAdvanced.checked
     ) then
    begin
    dispValue := Value;

    if (col = gccCypherKeysizeUnderlying) then
      begin
      if (strtoint(Value) = 0) then
        begin
        dispValue := '(null)';
        end
      else if (strtoint(Value) = -1) then
        begin
        dispValue := '(any)';
        end;
      end;

    if (col = gccCypherKeysizeRequired) then
      begin
      if (strtoint(Value) = 0) then
        begin
        dispValue := '(null)';
        end
      else if (strtoint(Value) = -1) then
        begin
        dispValue := '(any)';
        end;
      end;

    if (col = gccCypherBlockSize) then
      begin
      if (strtoint(Value) = -1) then
        begin
        dispValue := '(any)';
        end;
      end;

    item.SubItems.Add(dispValue);
    end;

end;

procedure TfrmGridReport_Cypher.SetupGrid();
var
  i: TGridColumn_Cypher;
begin
  inherited;

  AddCol('#');
  for i:=low(TGridColumn_Cypher) to high(TGridColumn_Cypher) do
    begin
    if (
        not(IsColumnAdvanced(ColumnToColIdx(i))) or
        ckShowAdvanced.checked
       ) then
      begin
      AddCol(TGridColumn_CypherTitle[i]);
      end;
    end;

end;

procedure TfrmGridReport_Cypher.PopulateGrid();
var
  allData: TFreeOTFECypherDriverArray;
  i: integer;
  j: integer;
  currDriver: TFreeOTFECypherDriver;
  currImpl: TFreeOTFECypher_v3;
  currRow: integer;
  item: TListItem;
begin
  inherited;

  if OTFEFreeOTFE.GetCypherDrivers(allData) then
    begin
    currRow := 0;
    for i:=low(allData) to high(allData) do
      begin
      currDriver := allData[i];
      for j:=low(currDriver.Cyphers) to high(currDriver.Cyphers) do
        begin
        inc(currRow);
        currImpl := currDriver.Cyphers[j];

        item := lvReport.Items.Insert(lvReport.Items.count);

        item.data := @currImpl;

        item.Caption := inttostr(currRow);

        AddSubItem(gccDriverTitle,             item, currDriver.Title);
        AddSubItem(gccDriverVersion,           item, OTFEFreeOTFE.VersionIDToStr(currDriver.VersionID));
        AddSubItem(gccDriverGUID,              item, GUIDToString(currDriver.DriverGUID));
        AddSubItem(gccDriverDeviceName,        item, currDriver.DeviceName);
        AddSubItem(gccDriverUserModeName,      item, currDriver.DeviceUserModeName);
        AddSubItem(gccDriverKernelModeName,    item, currDriver.DeviceKernelModeName);

        AddSubItem(gccCypherTitle,             item, currImpl.Title);
        AddSubItem(gccCypherKeysizeUnderlying, item, inttostr(currImpl.KeySizeUnderlying));
        AddSubItem(gccCypherMode,              item, FreeOTFECypherModeTitle[currImpl.Mode]);
        AddSubItem(gccCypherKeysizeRequired,   item, inttostr(currImpl.KeySizeRequired));
        AddSubItem(gccCypherBlocksize,         item, inttostr(currImpl.BlockSize));
        AddSubItem(gccCypherVersion,           item, OTFEFreeOTFE.VersionIDToStr(currImpl.VersionID));
        AddSubItem(gccCypherGUID,              item, GUIDToString(currImpl.CypherGUID));
        end;
      end;
    end;

  ResizeColumns();
end;

function TfrmGridReport_Cypher.IsColumnAdvanced(colIdx: integer): boolean;
begin
  Result := (ColIdxToColumn(colIdx) in ADVANCED_COLS_CYPHER);
end;

function TfrmGridReport_Cypher.CountDrivers(): integer;
var
  retval: integer;
  allData: TFreeOTFECypherDriverArray;
begin
  inherited;

  retval := 0;
  if OTFEFreeOTFE.GetCypherDrivers(allData) then
    begin
    retval := high(allData) - low(allData) + 1;
    end;

  Result := retval;
end;

function TfrmGridReport_Cypher.CountImplementations(): integer;
var
  retval: integer;
  allData: TFreeOTFECypherDriverArray;
  i: integer;
  currDriver: TFreeOTFECypherDriver;
begin
  inherited;

  retval := 0;
  if OTFEFreeOTFE.GetCypherDrivers(allData) then
    begin
    for i:=low(allData) to high(allData) do
      begin
      currDriver := allData[i];
      retval := retval + currDriver.CypherCount;
      end;
    end;

  Result := retval;
end;

procedure TfrmGridReport_Cypher.FormCreate(Sender: TObject);
begin
  inherited;

  self.caption := 'Available Cyphers';
  lblTitle.Caption := 'The following cyphers are available for use:';

end;

procedure TfrmGridReport_Cypher.FormShow(Sender: TObject);
begin
  inherited;

  if (CountImplementations() <= 0) then
    begin
    SDUMessageDlg(
                  'No cypher algorithms could be found.'+SDUCRLF+
                  SDUCRLF+
                  'Please start portable mode, or click "Drivers..." to install one or more cypher drivers',
                  mtError,
                  [mbOK],
                  0
                 );
    end;

end;

END.

